/* demo2.c
 * -------
 * Another demonstration of OpenMP4.0 constructs on the Zynq and the Epiphany.
 *
 * The OMPi team, 2015.
 */

#include <omp.h>
#include <stdio.h>


void demo_devices()
{
	int kernid = 0;

	/* Diagnostics */
	printf("Available devices:    %d\n", omp_get_num_devices());
	printf("Default device:       %d\n", omp_get_default_device());

	/* kernid is mapped to the device with initial value 0 */
	#pragma omp target data map(to:kernid)
	{
		int on_eCore, i;
		
		for(i=0; i<20; i++)
		{
			/* Kernels 0 & 10 will execute on the host */
			#pragma omp target map(from:on_eCore) if(i%10 != 0)
			{
				/* omp_is_initial_device returns TRUE only on Zynq */
				on_eCore = !omp_is_initial_device();
				kernid++;
			}

			/* Get value from the device */
			#pragma omp target update from(kernid)

			printf("Kernel %d executed on %s, kernid=%d\n", i, on_eCore ? "Epiphany" : "Zynq", kernid);

			/* If kernel was executed on the host */
			if(!on_eCore && i>0)
			{
				/* Reset kernid to 50 and push to the device */
				kernid = 50;
				#pragma omp target update to(kernid)
			}
		}

	}	
}


void main()
{
	demo_devices();
}

